package gregapi.data;

import gregapi.block.IBlockPlacable;
import gregapi.code.ItemStackContainer;
import gregapi.code.ItemStackMap;
import gregapi.item.ItemMetaTool;
import gregapi.network.INetworkHandler;
import gregapi.old.interfaces.internal.IGT_RecipeAdder;
import gregapi.tile.prefixblock.PrefixBlock;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;

import net.minecraft.block.Block;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.oredict.OreDictionary;

/**
 * @author Gregorius Techneticies
 * 
 * Class containing useful Constants and Lists.
 * 
 * Mainly made for use with static imports.
 * I am doing this to have a better Table alike view on my Code, so I can change things faster using the Block Selection Mode of eclipse.
 * 
 * For static imports in Eclipse go to "Window > Preferences > Java > Editor > Content Assist > Favorites" to set static importable Constant Classes such as this one as AutoCompleteable.
 */
public class CS {
	// unused: A, D, G, H, I, J, K, M, N, O, P, Q, R, S, T
	
	/** Because "true" and "false" are too long. Some Programmers might wanna kill me for that, but this looks much better than true and false, and also it is better to have something that is not 4 and 5 Characters long, because of symmetry */
	public static final boolean T = true, F = false;
	
	/** Character Set with all Numbers */
	public static final HashSet<Character> CN = new HashSet<Character>(Arrays.asList('0', '1', '2', '3', '4', '5', '6', '7', '8', '9'));
	/** Character Set with all lowercased Characters */
	public static final HashSet<Character> CL = new HashSet<Character>(Arrays.asList('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'));
	/** Character Set with all uppercased Characters */
	public static final HashSet<Character> CU = new HashSet<Character>(Arrays.asList('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'));
	
	/** The first 32 Bits */
	public static final int[] B = new int[] {1<<0,1<<1,1<<2,1<<3,1<<4,1<<5,1<<6,1<<7,1<<8,1<<9,1<<10,1<<11,1<<12,1<<13,1<<14,1<<15,1<<16,1<<17,1<<18,1<<19,1<<20,1<<21,1<<22,1<<23,1<<24,1<<25,1<<26,1<<27,1<<28,1<<29,1<<30,1<<31};
	
	/** 
	 * Renamed from "MATERIAL_UNIT" to just "U"
	 * 
	 * This is worth exactly one normal Item.
	 * This Constant can be divided by many commonly used Numbers such as 
	 * 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 20, 22, 24, 25, ... 64 or 81
	 * without loosing precision and is for that reason used as Unit of Amount.
	 * But it is also small enough to be multiplied with larger Numbers.
	 * 
	 * This is used to determine the amount of Material contained inside a prefixed Ore.
	 * For example Nugget = U / 9 as it contains out of 1/9th of an Ingot.
	 */
	public static final long U = 420076800;
	/** The Double Version of the Material Unit "U" */
	public static final double UD = U;
	
	/**
	 * Renamed from "FLUID_MATERIAL_UNIT" to just "L"
	 * 
	 * Fluid per Material Unit (Prime Factors: 3 * 3 * 2 * 2 * 2 * 2)
	 */
	public static final long L = 144;
	
	/** The offset between Celsius and Kelvin. */
	public static final long C = 273; // 273.15 IRL
	
	/** The Item WildCard Tag. Even shorter than the "-1" of the past */
	public static final short W = OreDictionary.WILDCARD_VALUE;
	
	/** The Voltage Tiers. Use this Array instead of the old named Voltage Variables */
	public static final long[] V = new long[] {8, 32, 128, 512, 2048, 8192, 32768, 131072, 524288, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE};
	
	/** The short Names for the Voltages */
	public static final String[] VN = new String[] {"ULV", "LV", "MV", "HV", "EV", "IV", "LuV", "ZPM", "UV", "MAX", "MAX", "MAX", "MAX", "MAX", "MAX", "MAX"};
	
	/** The long Names for the Voltages */
	public static final String[] VOLTAGE_NAMES = new String[] {"Ultra Low Voltage", "Low Voltage", "Medium Voltage", "High Voltage", "Extreme Voltage", "Insane Voltage", "Ludicrous Voltage", "ZPM Voltage", "Ultimate Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage"};
	
	/** The value of how many RF Units are worth an EU */
	public static int RF_PER_EU = 4;
	
	/** This way it is possible to have a Call Hierarchy of NullPointers in ItemStack based Functions, and also because most of the time I don't know what kind of Data Type the "null" stands for */
	public static final ItemStack NI = null;
	
	/** This way it is possible to have a Call Hierarchy of NullPointers in FluidStack based Functions, and also because most of the time I don't know what kind of Data Type the "null" stands for */
	public static final FluidStack NF = null;
	
	/** Not really Constants, but they set using the Config and therefore should be constant (those are for the Debug Modes) */
	public static boolean D1 = false, D2 = false, D3 = true, SHOW_HIDDEN_MATERIALS = false, SHOW_HIDDEN_PREFIXES = false, SHOW_HIDDEN_ITEMS = false;
	
	/** If you have to give something a World Parameter but there is no World... (Dummy World) */
	public static World DW;
	
	/** If you have to give something a World Parameter but there is no World... (Dummy World) */
	public static Object GT;
	
	/** Use this Object to add Recipes. (Recipe Adder) */
	public static IGT_RecipeAdder RA;
	
	/** For Internal Usage. */
	public static INetworkHandler NW_GT;
	
	/** For using the API internal Network Functionality for the already existing Packets. */
	public static INetworkHandler NW_API;
	
	/** Contains GT Blocks and Block related References. */
	public static class BlocksGT {
		public static PrefixBlock
		blockGem, blockDust, blockIngot, blockSolid, blockPlate,
		crateGtGem, crateGtDust, crateGtIngot, crateGtPlate;
		
		/** Ore Blocks */
		public static PrefixBlock
		ore			, oreSandstone			, oreNetherrack			, oreEndstone		, oreRedgranite			, oreBlackgranite		, oreUBIgneous[]		= new PrefixBlock[8], oreUBMetamorphic[]		= new PrefixBlock[8], oreUBSedimentary[]		= new PrefixBlock[8], oreGravel, oreSand, oreRedSand,
		oreBroken	, oreBrokenSandstone	, oreBrokenNetherrack	, oreBrokenEndstone	, oreBrokenRedgranite	, oreBrokenBlackgranite	, oreBrokenUBIgneous[]	= new PrefixBlock[8], oreBrokenUBMetamorphic[]	= new PrefixBlock[8], oreBrokenUBSedimentary[]	= new PrefixBlock[8],
		oreSmall	, oreSmallSandstone		, oreSmallNetherrack	, oreSmallEndstone	, oreSmallRedgranite	, oreSmallBlackgranite	, oreSmallUBIgneous[]	= new PrefixBlock[8], oreSmallUBMetamorphic[]	= new PrefixBlock[8], oreSmallUBSedimentary[]	= new PrefixBlock[8], oreSmallGravel, oreSmallSand, oreSmallRedSand;
		
		public static Block
		GraniteBlack, GraniteRed;
		
		/** Mappings for Stones to their correspondent Ores. No WildCard allowed! Register all 16 MetaData Values here instead. */
		public static Map<ItemStackContainer, IBlockPlacable>
		  stoneToNormalOres	= new ItemStackMap<ItemStackContainer, IBlockPlacable>()
		, stoneToBrokenOres	= new ItemStackMap<ItemStackContainer, IBlockPlacable>()
		, stoneToSmallOres	= new ItemStackMap<ItemStackContainer, IBlockPlacable>()
		;
	}
	
	/** Contains the Tool IDs for my MetaTool. */
	public static class ToolsGT {
		public static final short SWORD = 0, PICKAXE = 2, SHOVEL = 4, AXE = 6, HOE = 8, SAW = 10, HARDHAMMER = 12, SOFTHAMMER = 14, WRENCH = 16, FILE = 18, CROWBAR = 20, SCREWDRIVER = 22, MORTAR = 24, WIRECUTTER = 26, SCOOP = 28, BRANCHCUTTER = 30, UNIVERSALSPADE = 32, KNIFE = 34, BUTCHERYKNIFE = 36, SICKLE = 38, SENSE = 40, PLOW = 42, PLUNGER = 44, ROLLING_PIN = 46, DRILL_LV = 100, DRILL_MV = 102, DRILL_HV = 104, CHAINSAW_LV = 110, CHAINSAW_MV = 112, CHAINSAW_HV = 114, WRENCH_LV = 120, WRENCH_MV = 122, WRENCH_HV = 124, JACKHAMMER = 130, BUZZSAW = 140, SCREWDRIVER_LV = 150;
		public static ItemMetaTool sMetaTool;
	}
	
	/** Class Containing MOD ID Strings used in GT, since they are very common Parameters. */
	public static class ModIDs {
		/** MOD ID Strings */
		@SuppressWarnings("hiding")
		public static final String
	      GT				= "gregtech"
	    , API				= "gregapi"
	    
	    , IC2				= "IC2"
	    , NC				= "IC2NuclearControl"
	    
	    , TC				= "Thaumcraft"
	    
	    , TF				= "TwilightForest"
	    
	    , RC				= "Railcraft"
	    
	    , TE				= "ThermalExpansion"
	    
	    , AE 				= "appliedenergistics2"
	    
	    , TFC				= "terrafirmacraft"
	    
	    , UB				= "UndergroundBiomes"
	    , PFAA				= "PFAAGeologica"
	    
	    , FR				= "Forestry"
	    
	    , HaC				= "harvestcraft"
	    , APC				= "AppleCore"
	    , MaCr				= "magicalcrops"
	    
	    , GaEn				= "ganysend"
	    , GaSu				= "ganyssurface"
	    , GaNe				= "ganysnether"
	    
	    , BC				= "BuildCraft|Core"
	    , BC_CORE			= "BuildCraft|Core"
	    , BC_SILICON		= "BuildCraft|Silicon"
	    , BC_TRANSPORT		= "BuildCraft|Transport"
	    , BC_FACTORY		= "BuildCraft|Factory"
	    , BC_ENERGY			= "BuildCraft|Energy"
	    , BC_BUILDERS		= "BuildCraft|Builders"
	    
	    , GC				= "GalacticraftCore"
	    , GC_CORE			= "GalacticraftCore"
	    , GC_MARS			= "GalacticraftMars"
	    , GC_PLANETS		= "GalacticraftPlanets"
	    ;
	}
	
	/** File Paths and Resource Paths */
    public static final String
      TEX_DIR					= "textures/"
    , TEX_DIR_GUI				= TEX_DIR + "gui/"
    , TEX_DIR_ITEM				= TEX_DIR + "items/"
    , TEX_DIR_BLOCK				= TEX_DIR + "blocks/"
    , TEX_DIR_ENTITY			= TEX_DIR + "entity/"
    , TEX_DIR_ASPECTS			= TEX_DIR + "aspects/"
    
    , RES_PATH					= ModIDs.GT + ":" + TEX_DIR
    , RES_PATH_GUI				= ModIDs.GT + ":" + TEX_DIR_GUI
    , RES_PATH_ITEM				= ModIDs.GT + ":"
    , RES_PATH_BLOCK			= ModIDs.GT + ":"
    , RES_PATH_ENTITY			= ModIDs.GT + ":" + TEX_DIR_ENTITY
    , RES_PATH_ASPECTS			= ModIDs.GT + ":" + TEX_DIR_ASPECTS
    
    , RES_PATH_IC2				= ModIDs.IC2.toLowerCase() + ":"
    ;
}